;*******************************************************************************
;*
;*      title:	Firmware for the Kyub (Kyubmusic.com)
;*              
;*	date:	01/08/14	version:	0.3 (04/08/14)
;*	author:	WEIGU
;*
;*      Controller: ATmega32u4  Clock:          16 MHz
;*      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
;*     
;*      IN:     ADC0 (PF0), ADC1(PF1), ADC4(PF4)  analog
;*              PB0-3,PB7,PD0-2,PC6-7,PD5
;*      OUT:    PB4
;*      
;*      Copyright (c) 2014   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;       Einbinden der controllerspez.Definitionsdatei und Auswahl des Boards
;-------------------------------------------------------------------------------
.NOLIST                         ;List-Output ausschalten
;Teensy 2.0: ATmega32u4  16 MHz
.INCLUDE "m32u4def.inc"         ;AVR-Definitionsdatei einbinden
.EQU CRYSTAL = 16
.LIST                           ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
.ORG    SRAM_START
CTIME:  .BYTE 11                ;sensed charge time (11pads*1Byte)
CTIMEO: .BYTE 11                ;sensed charge time old value
ACDATA: .Byte 12                ;accelerator data
XMIN:   .Byte 1
XMAX:   .Byte 1
YMIN:   .Byte 1
YMAX:   .Byte 1
ZMIN:   .Byte 1
ZMAX:   .Byte 1

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG    0x0000                  ;Programm beginnt an der FLASH-Adresse 0x0000
RESET1: rjmp    INIT            ;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;       Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
        ;only needed for debugging!
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        rjmp    ISRGEN          ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        rjmp    ISRCOM          ;Springe zur ISR des EP COMmunication Interrupt

;-------------------------------------------------------------------------------
;       Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero    = r15           ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF    Tmp1    = r16           ;Register 16 dient als erster Zwischenspeicher
.DEF    Tmp2    = r17           ;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1    = r18           ;Register 18 dient als Zaehler
.DEF    Veloc   = r19           ;Velocity

.EQU    DPDDR   = DDRB          ;driverpin PB4 (Arduino: 13)
.EQU    DPPORT  = PORTB          
.EQU    DPNR    = 4
.EQU    MBDDR   = DDRD          ;modebutton PD4 (22)
.EQU    MBPORT  = PORTD  
.EQU    MBPIN   =  PIND   
.EQU    MBNR    = 4
.EQU    P0DDR   = DDRB          ;PAD 0  PB0 (0)
.EQU    P0PORT  = PORTB
.EQU    P0PIN   = PINB
.EQU    P0NR    = 0
.EQU    P1DDR   = DDRB          ;PAD 1  PB1 (1)
.EQU    P1PORT  = PORTB
.EQU    P1PIN   = PINB
.EQU    P1NR    = 1
.EQU    P2DDR   = DDRB          ;PAD 2  PB2 (2)
.EQU    P2PORT  = PORTB
.EQU    P2PIN   = PINB
.EQU    P2NR    = 2
.EQU    P3DDR   = DDRB          ;PAD 3  PB3 (3)
.EQU    P3PORT  = PORTB
.EQU    P3PIN   = PINB
.EQU    P3NR    = 3
.EQU    P4DDR   = DDRB          ;PAD 4  PB7 (0)
.EQU    P4PORT  = PORTB
.EQU    P4PIN   = PINB
.EQU    P4NR    = 7
.EQU    P5DDR   = DDRD          ;PAD 5  PD0 (0)
.EQU    P5PORT  = PORTD
.EQU    P5PIN   = PIND
.EQU    P5NR    = 0
.EQU    P6DDR   = DDRD          ;PAD 6  PD1 (0)
.EQU    P6PORT  = PORTD
.EQU    P6PIN   = PIND
.EQU    P6NR    = 1
.EQU    P7DDR   = DDRD          ;PAD 7  PD2 (0)
.EQU    P7PORT  = PORTD
.EQU    P7PIN   = PIND
.EQU    P7NR    = 2
.EQU    P8DDR   = DDRD          ;PAD 8  PD5 (23) (PD3 = TX = Midi Out)
.EQU    P8PORT  = PORTD
.EQU    P8PIN   = PIND
.EQU    P8NR    = 5
.EQU    P9DDR   = DDRC          ;PAD 9  PC6 (0)
.EQU    P9PORT  = PORTC
.EQU    P9PIN   = PINC
.EQU    P9NR    = 6
.EQU    P10DDR  = DDRC          ;PAD 10 PC7 (0)
.EQU    P10PORT = PORTC
.EQU    P10PIN  = PINC
.EQU    P10NR   = 7

.EQU    Thresh  = 20            ;Threshhold for capacitive touch
.EQU    Channel = 0             ;Midi Channel: 0 = 1

        ;stack
        ldi     Tmp1,HIGH(RAMEND) 
        out     SPH,Tmp1          
        ldi     Tmp1,LOW(RAMEND)
        out     SPL,Tmp1

        ;reset Fusebit CKDIV8 (do not divide clock frequency)
        ldi     Tmp1,0x80        ;CLKPCE = 1 override Bit CKDIV
        sts     CLKPR,Tmp1       
        clr     Tmp1             ;CLKPR = 0, divisor = 1
        sts     CLKPR,Tmp1

        ;ADC
        ldi     Tmp1,0b01100000 ;REFS = 01 REF = AVCC
        sts     ADMUX,Tmp1      ;ADLAR = 1 8 Bit
                                ;MUX = 00000 ADC0
        ldi     Tmp1,0b10000100 ;ADEN = 1 (ADC on)
        sts     ADCSRA,Tmp1     ;ADPS = 100 16MHz/16 = 1MHz
        
        ;output                        
        sbi     DPDDR,DPNR      ;driverpin
        ;input
        cbi     P0DDR,P0NR      ;PAD 0
        cbi     P1DDR,P1NR      ;PAD 1
        cbi     P2DDR,P2NR      ;PAD 2
        cbi     P3DDR,P3NR      ;PAD 3
        cbi     P4DDR,P4NR      ;PAD 4
        cbi     P5DDR,P5NR      ;PAD 5
        cbi     P6DDR,P6NR      ;PAD 6
        cbi     P7DDR,P7NR      ;PAD 7
        cbi     P8DDR,P8NR      ;PAD 8 
        cbi     P9DDR,P9NR      ;PAD 9
        cbi     P10DDR,P10NR    ;PAD 10
        cbi     MBDDR,MBNR      ;modebutton
        sbi     MBPORT,MBNR     ;modebutton pull-up
        
        ;clear charge time old table
        ldi     Cnt1,11
        ldi     XL,LOW(CTIMEO)
        ldi     XH,HIGH(CTIMEO)
INIL1:  st      X+,Zero
        dec     Cnt1
        brne    INIL1
        
        rcall   E232INI         ;for Midi
        
        ;USB: only needed for debugging!
        rcall   USBINI          
        ldi     Tmp1,12         ;n Bytes over USB
        sts     EP1_CNT,Tmp1

;-------------------------------------------------------------------------------
;        Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   ;get charge time (11 times)
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR     ;driverpin high start charging
ML20:   sbic    P0PIN,P0NR      ;(2T) 6T=6/16us = 0,375us*256: overflow at 95us
        rjmp    MAIN21
        inc     Tmp1            ;(1T)
        cpi     Tmp1,255        ;(1T)
        brne    ML20            ;(2T)
MAIN21: sts     CTIME+0,Tmp1    ;save charge time
        sei
        cbi     DPPORT,DPNR     ;driverpin low start discharging        

        ldi     Tmp1,0b01100000 ;ADC0 x
        sts     ADMUX,Tmp1
        rcall   W100us          ;wait to discharge
        lds     Tmp1,ADCSRA     ;start AD conversion ADSC = 1
        ori     Tmp1,0x40       
        sts     ADCSRA,Tmp1
ADPOL1: lds     Tmp1,ADCSRA     ;polling
        sbrc    Tmp1,ADSC
        rjmp    ADPOL1
        lds     Tmp1,ADCH        
        sts     XMIN,Tmp1
        sts     XMAX,Tmp1
        sts     ACDATA+0+0,Tmp1
        
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML21:   sbic    P1PIN,P1NR
        rjmp    MAIN22
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML21
MAIN22: sts     CTIME+1,Tmp1
        sei
        cbi     DPPORT,DPNR

        ldi     Tmp1,0b01100001 ;ADC1 y
        sts     ADMUX,Tmp1
        rcall   W100us
        lds     Tmp1,ADCSRA     
        ori     Tmp1,0x40       
        sts     ADCSRA,Tmp1
ADPOL2: lds     Tmp1,ADCSRA     
        sbrc    Tmp1,ADSC
        rjmp    ADPOL2
        lds     Tmp1,ADCH 
        sts     YMIN,Tmp1
        sts     YMAX,Tmp1
        sts     ACDATA+0+4,Tmp1

        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML22:   sbic    P2PIN,P2NR
        rjmp    MAIN23
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML22
MAIN23: sts     CTIME+2,Tmp1
        sei
        cbi     DPPORT,DPNR

        ldi     Tmp1,0b01100100 ;ADC4 z
        sts     ADMUX,Tmp1
        rcall   W100us
        lds     Tmp1,ADCSRA     
        ori     Tmp1,0x40       
        sts     ADCSRA,Tmp1
ADPOL3: lds     Tmp1,ADCSRA     
        sbrc    Tmp1,ADSC
        rjmp    ADPOL3
        lds     Tmp1,ADCH
        sts     ZMIN,Tmp1
        sts     ZMAX,Tmp1

        sts     ACDATA+0+8,Tmp1
        
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML23:   sbic    P3PIN,P3NR
        rjmp    MAIN24
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML23
MAIN24: sts     CTIME+3,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML24:   sbic    P4PIN,P4NR
        rjmp    MAIN25
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML24
MAIN25: sts     CTIME+4,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML25:   sbic    P5PIN,P5NR
        rjmp    MAIN26
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML25
MAIN26: sts     CTIME+5,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML26:   sbic    P6PIN,P6NR
        rjmp    MAIN27
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML26
MAIN27: sts     CTIME+6,Tmp1
        sei
        cbi     DPPORT,DPNR
        sts     ADMUX,Tmp1
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML27:   sbic    P7PIN,P7NR
        rjmp    MAIN28
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML27
MAIN28: sts     CTIME+7,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML28:   sbic    P8PIN,P8NR
        rjmp    MAIN29
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML28
MAIN29: sts     CTIME+8,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML29:   sbic    P9PIN,P9NR
        rjmp    MAIN2A
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML29
MAIN2A: sts     CTIME+9,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us
        cli
        clr     Tmp1
        sbi     DPPORT,DPNR
ML2A:   sbic    P10PIN,P10NR
        rjmp    MAIN3
        inc     Tmp1
        cpi     Tmp1,255
        brne    ML2A
MAIN3:  sts     CTIME+10,Tmp1
        sei
        cbi     DPPORT,DPNR
        rcall   W100us

        ;MIDI send NoteOn, NoteOff
        ldi     Cnt1,11
        ldi     XL,LOW(CTIME)
        ldi     XH,HIGH(CTIME)
        ldi     YL,LOW(CTIMEO)
        ldi     YH,HIGH(CTIMEO)
        ldi     ZL,LOW(PADNOTE1*2)
        ldi     ZH,HIGH(PADNOTE1*2)
MMID1L: ld      Tmp1,X+
        ld      Tmp2,Y+
        cpi     Tmp1,Thresh
        brsh    MMID11
        rjmp    MMID1
MMID11: cpi     Tmp2,Thresh     ;pad touched
        brlo    MMID1V
        rjmp    MMID2           ;if pad still touched do nothing else
        ;calculate velocity
MMID1V: cpi     Cnt1,11-3       ;y
        breq    MMIDY1
        cpi     Cnt1,11-4
        breq    MMIDY1
        cpi     Cnt1,11-1       ;z
        breq    MMIDZ1
        cpi     Cnt1,11-2
        breq    MMIDZ1
        cpi     Cnt1,11-5
        breq    MMIDZ1
        cpi     Cnt1,11-6
        breq    MMIDZ1
        ;x, get more samples from accelerometer an calculate velocity
MMIDX1: ldi     Tmp1,0b01100000 ;ADC0 x 
        sts     ADMUX,Tmp1
        rcall   W100us
        lds     r24,XMIN
        lds     r25,XMAX
        rcall   CALVEL
        mov     Veloc,r24
        rjmp    MMIDON
MMIDY1: ldi     Tmp1,0b01100001 ;ADC1 y 
        sts     ADMUX,Tmp1
        rcall   W100us
        lds     r24,YMIN
        lds     r25,YMAX
        rcall   CALVEL
        mov     Veloc,r24
        rjmp    MMIDON
MMIDZ1: ldi     Tmp1,0b01100100 ;ADC4 z 
        sts     ADMUX,Tmp1
        rcall   W100us
        lds     r24,ZMIN
        lds     r25,ZMAX
        rcall   CALVEL
        mov     Veloc,r24

        ;send NoteOn
MMIDON: ldi     r24,0x90
        ori     r24,Channel
        rcall   E232SND
        lpm     r24,Z
        rcall   E232SND
        mov     r24,Veloc
        sts     EP1_BUF+11,Veloc ;debugging
        cpi     r24,16
        brsh    MMIDO1
        lsl     r24     ;x4       
        lsl     r24
        cpi     r24,127
        brsh    MMIDO1
        cpi     r24,2
        brsh    MMIDO2
        ldi     r24,2
        rjmp    MMIDO2
MMIDO1: ldi     r24,127
        rjmp    MMIDO2
MMIDO2: rcall   E232SND
        rjmp    MMID2
MMID1:  cpi     Tmp2,Thresh     ;pad not touched
        brlo    MMID2           ;if pad still not touched do nothing else
        ldi     r24,0x80        ;send NoteOff
        ori     r24,Channel
        rcall   E232SND
        lpm     r24,Z
        rcall   E232SND
        clr     Veloc
        sts     EP1_BUF+11,Veloc ;debugging
        mov     r24,Veloc
        rcall   E232SND
MMID2:  adiw    ZL,1
        dec     Cnt1
        breq    MMID3
        rjmp    MMID1L
        ;copy CTIME to CTIMEO old
MMID3:  ldi     Cnt1,11
        ldi     XL,LOW(CTIME)
        ldi     XH,HIGH(CTIME)
        ldi     YL,LOW(CTIMEO)
MMID2L: ldi     YH,HIGH(CTIMEO)
        ld      Tmp1,X+
        st      Y+,Tmp1
        dec     Cnt1
        brne    MMID2L

        
        
        ;fill USB Buffer and send (only needed for debugging
        ldi     Cnt1,11
        ldi     XL,LOW(CTIME)
        ldi     XH,HIGH(CTIME)
        ldi     YL,LOW(EP1_BUF)
        ldi     YH,HIGH(EP1_BUF)
MUSB1L: ld      Tmp1,X+
        st      Y+,Tmp1
        dec     Cnt1
        brne    MUSB1L
        
        ldi     Tmp1,1          ;set EP1 Flag to send data
        sts     EP1_FLAG,Tmp1
        
        rjmp    MAIN            ;Endlosschleife

;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;Max:Min values in r25:r24
;velocity in r24
CALVEL: push    r16
        push    r18
        
        ldi     r18,250         ;250*15us = 4ms
CALVL1: lds     Tmp1,ADCSRA     ;start conversion
        ori     Tmp1,0x40       
        sts     ADCSRA,Tmp1
CALVL2: lds     Tmp1,ADCSRA     ;polling 
        sbrc    Tmp1,ADSC
        rjmp    CALVL2
        lds     r16,ADCH
        cp      r16,r24
        brsh    CALVE1
        mov     r24,r16
CALVE1: cp      r16,r25
        brlo    CALVE2
        mov     r25,r16
CALVE2: dec     r18
        brne    CALVL1
        sub     r25,r24
        mov     r24,r25
        
        pop     r18
        pop     r16
        ret


.INCLUDE "lib/SR_TIME_16M.asm"          ;Time lib
.INCLUDE "lib/SR_USB_HID_V1_2.asm"      ;USB HID lib
.INCLUDE "lib/SR_EIA232.asm"            ;USART lib

;-------------------------------------------------------------------------------
;       Flash-Tabellen
;-------------------------------------------------------------------------------

PADNOTE1:
.DB     60,62,64,65,67,69,71,72,74,76,77        ;Do majeur (C major)

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT                           ;Ende des Quelltextes