;*******************************************************************************
;*
;*      Titel:  Unterprogramme zur Nutzung der seriellen Schnittstelle
;*              (SR_EIA232.asm)
;*
;*      Datum:  04/02/12        Version:        0.6 (27_05_14)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A/644p/32u4	Quarzfrequenz:  
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*      
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;Damit der Code im Flash gering bleibt werden ausschliesslich die benutzten
;Unterprogramme freigeschaltet (Flag = 1)
.EQU    Flag_E232INI  = 1	;Schnittstelle initialisieren (default = 1)
.EQU    Flag_E232INI2 = 0	;2 Schnittstelle initialisieren (mega644p)
.EQU    Flag_E232SND  = 1	;Polling Senden
.EQU    Flag_E232SND2 = 0	;Polling Senden 2 Schnittstelle (mega644p)
.EQU    Flag_E232RCV  = 0	;Polling Empfang
.EQU    Flag_E232RCV2 = 0	;Polling Empfang 2 Schnittstelle (mega644p)
.EQU    Flag_E232RCT  = 0	;Polling Empfang mit Timeout
.EQU    Flag_E232RCT2 = 0	;Polling Empfang  mit Timeout 2 Schnittstelle
.EQU    Flag_E232HEX  = 0	;Senden Hexzahl
.EQU    Flag_E232HEX2 = 0	;Senden Hexzahl 2 Schnittstelle (mega644p)
.EQU    Flag_E232DEZ  = 0       ;Senden Dezimal (SR_MATH muss eingebunden sein)
.EQU    Flag_ISRTX    = 0	;Interrupt Senden
.EQU    Flag_ISRTX2   = 0	;Interrupt Senden 2 Schnittstelle (mega644p)
.EQU    Flag_ISRRX    = 0	;Interrupt Empfang
.EQU    Flag_ISRRX2   = 0	;Interrupt Empfang 2 Schnittstelle (mega644p)
.EQU    Flag_E232CL   = 0	;"command list" ausgeben
.EQU    Flag_E232FL   = 1	;Leeren des Empfangsbuffer (flush)
.EQU    Flag_E232FL2  = 0	;Leeren Empfangsbuff. 2 Schnittstelle (mega644p)

;///////////////////////////////////////////////////////////////////////////////
;	Makros 
;	siehe http://www.mikrocontroller.net/articles/AVR_Assembler_Makros
;///////////////////////////////////////////////////////////////////////////////
;Bei neuen Chips liegen viele SF-Register ueber den 64 Adressen die mit den 
;Befehlen "in" und "out" zu erreichen sind. Fuer sie muessen "sts" und "lds"
;verwendet werden. Dies wird mit den folgenden beiden Makros erledigt. Statt der
;Befehle "in" und "out" werden in der Bibliothek die Makros "input" und "output"
;verwendet.

;Einlesen eines Ports mit input Arbeitsregister,SF-Register
.IFNDEF MACROINPUT
.SET	MACROINPUT = 1
  .MACRO input
    .IF @1 < 64
	in	@0, @1
    .ELSE
	lds	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;Schreiben eines Ports mit output SF-Register,Arbeitsregister
.IFNDEF MACROOUTPUT
.SET	MACROOUTPUT = 1
  .MACRO output
    .IF @0 < 64
	out	@0, @1
    .ELSE
  	sts	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
RXFLAG:	 .BYTE 1		;wird von der Routine ISRRX benoetigt	
RXFLAG2: .BYTE 1		;wird von der Routine ISRRX2 benoetigt	
.CSEG

; Achtung Datenblatt beachten!
; Nicht alle Kombinationen liefern eine fehlerfreie Uebertragung
; Bei 16 MHz sind 9600, 19200 und 38400 Bit/s gute Werte
.EQU    EIA_Baud    = 31250	;Bitrate
.EQU    EIA_Baud2   = 9600      ;Bitrate 2 Schnittstelle (mega644p)
.EQU    E232TIMEOUT = 1000      ;Timeout*1ms fuer E232RCT
.EQU    E232TIMEOUT2 = 1000     ;Timeout*1ms fuer E232RCT2

.IFNDEF CPU_Clock
.EQU    CPU_Clock = 16000000    ;Systemtakt
.ENDIF

;ATmega32U4  !!! Achtung 1 serielle Schnittstelle USART1 an PD2 und PD3 !!!
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x95) && (SIGNATURE_002 == 0x87))
.EQU    UCSRA   = UCSR1A
.EQU    UCSRB   = UCSR1B
.EQU    UCSRC   = UCSR1C
.EQU    UBRRL   = UBRR1L
.EQU    UBRRH   = UBRR1H
.EQU    UDR     = UDR1
.EQU    UDRE    = UDRE1
.EQU    RXC     = RXC1
.EQU    TXEN    = TXEN1
.EQU    RXEN    = RXEN1
.EQU    UDRIE   = UDRIE1
.EQU    RXCIE  =  RXCIE1
.ENDIF

;ATmega644P  !!! 2 serielle Schnittstelle USART0 an PD0 und PD1 (USART1 PD2+3)!!
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x96) && (SIGNATURE_002 == 0x0A))
.EQU    UCSRA   = UCSR0A
.EQU    UCSRB   = UCSR0B
.EQU    UCSRC   = UCSR0C
.EQU    UBRRL   = UBRR0L
.EQU    UBRRH   = UBRR0H
.EQU    UDR     = UDR0
.EQU    UDRE    = UDRE0
.EQU    RXC     = RXC0
.EQU    TXEN    = TXEN0
.EQU    RXEN    = RXEN0
.EQU    UDRIE   = UDRIE0
.EQU    RXCIE   = RXCIE0
.ENDIF

;-------------------------------------------------------------------------------
;       EIA 232 (USART) initialisieren (Datenformat 8N1)
;-------------------------------------------------------------------------------
.IF Flag_E232INI == 1
E232INI: push	r16
	;Sender und Empfaenger einschalten und Datenformat festlegen
.IF ((Flag_ISRTX == 1) || (Flag_E232SND == 1))
	input   r16,UCSRB
        ori	r16,0x08        ;Sender einschalten (TxEnable, Bit 3)
        output  UCSRB,r16        	
.ENDIF
.IF ((Flag_ISRRX == 1) || (Flag_E232RCV  == 1))
        input   r16,UCSRB
        ori	r16,0x10	;Empfaenger einschalten (RxEnable Bit 4)
        output  UCSRB,r16        
.ENDIF
.IF Flag_ISRRX == 1                
        input   r16,UCSRB
        ori	r16,0x80	;Empfangsinterrupt (RXCIE) freigeben
        output  UCSRB,r16        			
.ENDIF	
        ldi     r16,0x86        ;UCSRC = 0b10000110
        output  UCSRC,r16       ;Waehle Asynchr. 8N1
                                ;B7 URSEL=1 UCSRC ausgewaehlt
                                ;B6 UMSEL=0 Asynchron
                                ;B45 UPM=00 keine Paritaet
                                ;B3 USBS=0 1 Stoppbit
                                ;B21 UCSZ=11 8 Datenbit
                                ;B0 UCPOL=0 bei Asyn.
        ;Baudrate initialisieren (Teiler fuer Baudrate festlegen)
        ldi     r16,HIGH((CPU_Clock/(EIA_Baud*16))-1)        
        output  UBRRH,r16              
        ldi     r16,LOW((CPU_Clock/(EIA_Baud*16))-1)
        output  UBRRL,r16  
        
.IF Flag_ISRRX == 1        
	sts	RXFLAG,Zero	;Flag fuer die Routine ISRRX loeschen	
.ENDIF                
        pop	r16
        ret
.ENDIF

;-------------------------------------------------------------------------------
;       EIA 232 (USART) initialisieren (8N1) 2 Schnittstelle (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232INI2 == 1
E232INI2: push	r16
	;Sender und Empfaenger einschalten und Datenformat festlegen
.IF ((Flag_ISRTX2 == 1) || (Flag_E232SND2 == 1))
	input   r16,UCSR1B
        ori	r16,0x08        ;Sender einschalten (TxEnable, Bit 3)
        output  UCSR1B,r16        	
.ENDIF
.IF ((Flag_ISRRX2 == 1) || (Flag_E232RCV2  == 1))
        input   r16,UCSR1B
        ori	r16,0x10	;Empfaenger einschalten (RxEnable Bit 4)
        output  UCSR1B,r16        
.ENDIF
.IF Flag_ISRRX2 == 1                
        input   r16,UCSR1B
        ori	r16,0x80	;Empfangsinterrupt (RXCIE) freigeben
        output  UCSR1B,r16        			
.ENDIF
	ldi     r16,0x06        ;UCSRC = 10000110b
        output  UCSR1C,r16      ;Waehle Asynchr. 8N1
                                ;B7 URSEL=1 UCSRC ausgewaehlt
                                ;B6 UMSEL=0 Asynchron
                                ;B45 UPM=00 keine Paritaet
                                ;B3 USBS=0 1 Stoppbit
                                ;B21 UCSZ=11 8 Datenbit
                                ;B0 UCPOL=0 bei Asyn.
        ;Baudrate initialisieren (Teiler fuer Baudrate festlegen)
        ldi     r16,HIGH((CPU_Clock/(EIA_Baud2*16))-1)        
        output  UBRR1H,r16              
        ldi     r16,LOW((CPU_Clock/(EIA_Baud2*16))-1)
        output  UBRR1L,r16  
        
.IF Flag_ISRRX2 == 1        
	sts	RXFLAG2,Zero	;Flag fuer die Routine ISRRX loeschen	
.ENDIF                
        pop	r16
        ret
.ENDIF

;===============================================================================
;       Polling
;===============================================================================
;-------------------------------------------------------------------------------
;       Senden eines Zeichens (Parameter r24)
;-------------------------------------------------------------------------------
.IF Flag_E232SND == 1
E232SND: push	r16 
E232SL: input	r16,UCSRA	;Warte bis UDREmpty gesetzt ist, da der Schreib-
	sbrs	r16,UDRE	;versuch sonst ignoriert wird        	
        rjmp    E232SL         
        output  UDR,r24         ;und gib das Zeichen aus        
        pop	r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Senden eines Zeichens (Parameter r24) 2 Schnittstelle (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232SND2 == 1
E232SND2: push	r16 
E232SL2: input	r16,UCSR1A	;Warte bis UDREmpty gesetzt ist, da der Schreib-
	sbrs	r16,UDRE1	;versuch sonst ignoriert wird        	
        rjmp    E232SL2         
        output  UDR1,r24         ;und gib das Zeichen aus        
        pop	r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Empfang eines Zeichens (Zeichen r24, Fehler in r25) 2 Schnittstelle
;       (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232RCV == 1
E232RCV: input r25,UCSRA	;Warte bis RXC (Receive Complete) gesetzt
	sbrs	r25,RXC		
        rjmp    E232RCV
        input   r25,UCSRA       ;UCSRA einlesen und Fehlerflags maskieren
        andi    r25,0b00011100  
	input	r24,UDR		;Zeichen einlesen
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Empfang eines Zeichens (Zeichen r24, Fehler in r25) 2 Schnittstelle
;       (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232RCV2 == 1
E232RCV2: input r25,UCSR1A	;Warte bis RXC (Receive Complete) gesetzt
	sbrs	r25,RXC1	
        rjmp    E232RCV2
        input   r25,UCSR1A      ;UCSR1A einlesen und Fehlerflags maskieren
        andi    r25,0b00011100  
	input	r24,UDR1	;Zeichen einlesen
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Empfang eines Zeichens mit TimeOut
;       (Zeichen r24, Fehler in r25)
;-------------------------------------------------------------------------------
.IF Flag_E232RCT == 1
E232RCT: push   r16
        push    r18

        ldi     r25,HIGH(E232TIMEOUT)
        ldi     r24,LOW(E232TIMEOUT)    
        ldi     r18,222         ;about 1ms
E232RCL: rcall  W1us
        rcall   W1us
        rcall   W1us
        rcall   W1us
        dec     r18
        brne    E232RC1
        ldi     r18,222
        sbiw    r24,1
        breq    E232RC2
E232RC1: input r16,UCSRA        ;Warte bis RXC (Receive Complete) gesetzt
        sbrs    r16,RXC         
        rjmp    E232RCL
        input   r25,UCSRA       ;UCSRA einlesen und Fehlerflags maskieren
        andi    r25,0b00011100  
        input   r24,UDR         ;Zeichen einlesen
        rjmp    E232RCR
E232RC2: ldi    r25,'T'         ;Fehler Timeout, r24 nicht gueltig!

E232RCR: pop    r18
        pop     r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Empfang eines Zeichens mit TimeOut
;       (Zeichen r24, Fehler in r25) 2 Schnittstelle (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232RCT2 == 1
E232RCT2: push   r16
        push    r18

        ldi     r25,HIGH(E232TIMEOUT2)
        ldi     r24,LOW(E232TIMEOUT2)    
        ldi     r18,222         ;about 1ms
E232RCL2: rcall  W1us
        rcall   W1us
        rcall   W1us
        rcall   W1us
        dec     r18
        brne    E232RC12
        ldi     r18,222
        sbiw    r24,1
        breq    E232RC22
E232RC12: input r16,UCSRA1      ;Warte bis RXC (Receive Complete) gesetzt
        sbrs    r16,RXC1
        rjmp    E232RCL2
        input   r25,UCSRA1      ;UCSRA einlesen und Fehlerflags maskieren
        andi    r25,0b00011100  
        input   r24,UDR1        ;Zeichen einlesen
        rjmp    E232RCR2
E232RC22: ldi    r25,'T'        ;Fehler Timeout, r24 nicht gueltig!

E232RCR2: pop    r18
        pop     r16
        ret
.ENDIF

;-------------------------------------------------------------------------------
;       Senden eines Bytes in Hexadezimal (Parameter r24)
;       Ausgabe: 2 Zeichen+Space
;-------------------------------------------------------------------------------        
.IF Flag_E232HEX == 1
E232HEX:
	push	r16	
	push	r17
	push	r24
	push	r25
	
	ldi	r16,0x30	;ASCII Offset Ziffer
	ldi	r17,0x37	;ASCII Offset Grossbuchstabe
	
	mov	r25,r24
	andi	r24,0xF0
	swap	r24	
	cpi	r24,10
	brge	E232H1
	add	r24,r16	
	rjmp	E232H2
E232H1:	add	r24,r17	
E232H2:	rcall	E232SND	
	mov	r24,r25
	andi	r24,0x0F
	cpi	r24,10
	brge	E232H3
	add	r24,r16	
	rjmp	E232H4
E232H3:	add	r24,r17		
E232H4:	rcall	E232SND	
	ldi	r24,' '
	rcall	E232SND
	
	pop	r25
	pop	r24
	pop	r17
	pop	r16
	ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Senden eines Bytes in Dezimal (Parameter r24)
;       Ausgabe: 3 Zeichen+Space
;       Bibliothek SR_MATH muss eingebunden sein
;-------------------------------------------------------------------------------        
.IF Flag_E232DEZ == 1
E232DEZ: push    r24
        rcall   DEC2A3
        lds     r24,TDEC2A+0
        rcall   E232SND 
        lds     r24,TDEC2A+0
        rcall   E232SND 
        lds     r24,TDEC2A+0
        rcall   E232SND 
        ldi     r24,' '
        rcall   E232SND 
	pop	r24
	ret
.ENDIF        

;===============================================================================
;       Interrupt
;===============================================================================
;-------------------------------------------------------------------------------
;	Interrupt Routine zum Senden eines Zeichens ueber EIA232
;	
;	Im Hauptprogramm muss der Interruptvektor initialisiert werden:
;	.ORG	UDREaddr (bzw. UDRE1addr)
;		rjmp	ISRTX
;	
;	E232INI muss in der Initialisierungsphase aufgerufen werden
;		rcall	E232INI
;		
;	Das spezifische Interrupt muss kurz vor dem Senden freigeschaltet
;	werden. Der Text befindet sich in einer Flash-Tabelle und schliesst
;	mit dem Nullbyte ab. Die Tabelle muss ensprechend initialisiert werden.
;	
;		ldi	ZL,LOW(TEXT*2)	;Tabellenzeiger initialisieren
;		ldi	ZH,HIGH(TEXT*2)
;		input   r16,UCSRB	;Interrupt (UDRIE) freischalten
;       	ori	r16,0x20         
;       	output  UCSRB,r16        	
;	TEXT:	.DB	"Hallo",0
;-------------------------------------------------------------------------------
.IF Flag_ISRTX == 1
ISRTX:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;

	lpm	r16,Z+		;Zeichen aus der Tabelle einlesen
	cpi	r16,0		;Falls Nullbyte, dann raus
	breq	ISRTXC			
	output	UDR,r16		;Zeichen ausgeben
	rjmp	ISRTXR
	
ISRTXC:	input   r16,UCSRB	;Interrupt sperren (UDRIE loeschen)
        andi	r16,0xDF         
       	output  UCSRB,r16        	

ISRTXR:	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti    		;zurueck und Interrupts wieder freigeben
.ENDIF    

;-------------------------------------------------------------------------------
;	Interrupt Routine zum Empfang von Zeichen (Befehlen) ueber EIA232
;	
;	;	Im Hauptprogramm muss der Interruptvektor initialisiert werden:
;	.ORG	URXCaddr (bzw.URXC1addr)
;		rjmp	ISRRX
;	
;	E232INI muss in der Initialisierungsphase aufgerufen werden
;		rcall	E232INI
;	
;	Das obere Nibble des Zeichenzaehlers (RXCFLAG im SRAM) definiert den
;	Befehl (1-15), das untere Nibble das momentan empfangene Zeichen des
;	Befehls. Befehle muessen mit unterschiedlichen Buchstaben beginnen!
;	Ist der Befehl vollstaendig empfangen worden, wird 0xF ins untere Nibble
;	geschrieben. Dies dient als Flag im Hauptprogramm um den Befehl zu
;	verarbeiten.
;	Die Routine muss natuerlich entsprechend den Anforderungen veraendert
;	werden
;	
;	Beispielroutine:
;	1 Befehl: "?" fuer help
;	2 Befehl: "g" fuer "get data" Die aktuellen Daten sollen gesendet werden          
;	3 Befehl: "a" fuer "all data" Alle Daten des EEPROMS sollen
;		                      zurueckgesendet werden         
;	          
;-------------------------------------------------------------------------------
.IF Flag_ISRRX == 1
ISRRX:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;
	push    r17    
		
	input   r16,UDR         ;Zeichen einlesen
	;output	UDR,r16		;echo zum Testen

	lds	r17,RXFLAG
	cpi	r17,0x00
	brne	ISRRX0
	cpi	r16,'?'		;Sortiere den Befehl
	breq	ISRRX1
	cpi	r16,'g'	
	breq	ISRRX2
	cpi	r16,'a'	
	breq	ISRRX3
	cpi	r16,'t'	
	brne	ISRRX0R		;raus
	rjmp    ISRRX4  
	
ISRRX0: andi	r17,0xF0	;falls ein Befehl mehr als 1 Buchstabe	
	cpi	r17,0x40	;enthaelt!
	brne 	ISRRX0R
	rjmp	ISRRX41
ISRRX0R: rjmp	ISRRXR		;raus

        ;1 Befehl
ISRRX1:	ldi	r17,0x1F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;2 Befehl
ISRRX2:	ldi	r17,0x2F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;3 Befehl
ISRRX3:	ldi	r17,0x3F	
	sts	RXFLAG,r17	
	rjmp	ISRRXR		;raus

	;4 Befehl
ISRRX4:	ldi	r17,0x41
	sts	RXFLAG,r17
	rjmp	ISRRXR
ISRRX41: cpi	r16,' '	
	brne	ISRRX42
	lds	r17,RXFLAG
	cpi	r17,0x41
	brne	ISRRX42	
	inc	r17	
	sts	RXFLAG,r17
	rjmp	ISRRXR
	
	;Tag
ISRRX42: lds	r17,RXFLAG
	cpi	r17,0x42
	brne	ISRRX43
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+4,r16
	rjmp	ISRRXR

ISRRX43: lds	r17,RXFLAG
	cpi	r17,0x43
	brne	ISRRX44
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+4
	add	r16,r17
        sts     RTCTAB+4,r16
        rjmp	ISRRXR

	;Monat
ISRRX44: lds	r17,RXFLAG
	cpi	r17,0x44
	brne	ISRRX45
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+5,r16
	rjmp	ISRRXR

ISRRX45: lds	r17,RXFLAG
	cpi	r17,0x45
	brne	ISRRX46
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+5
	add	r16,r17
        sts     RTCTAB+5,r16
        rjmp	ISRRXR

        ;Jahr
ISRRX46: lds	r17,RXFLAG
	cpi	r17,0x46
	brne	ISRRX47
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+6,r16
 	rjmp	ISRRXR

ISRRX47: lds	r17,RXFLAG
	cpi	r17,0x47
	brne	ISRRX48
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+6
	add	r16,r17
        sts     RTCTAB+6,r16
        rjmp	ISRRXR

        ;Stunden
ISRRX48: lds	r17,RXFLAG
	cpi	r17,0x48
	brne	ISRRX49
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+2,r16
	rjmp	ISRRXR

ISRRX49: lds	r17,RXFLAG
	cpi	r17,0x49
	brne	ISRRX4A
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+2
	add	r16,r17
        sts     RTCTAB+2,r16
        rjmp	ISRRXR
        
        ;Minuten
ISRRX4A: lds	r17,RXFLAG
	cpi	r17,0x4A
	brne	ISRRX4B
	lds	r17,RXFLAG	;inc RXFLAG
	inc	r17		
	sts	RXFLAG,r17
	subi	r16,0x30
	swap	r16
        sts     RTCTAB+1,r16
 	rjmp	ISRRXR

ISRRX4B: lds	r17,RXFLAG
	cpi	r17,0x4B
	brne	ISRRXR	
;	ldi	r17,0x4F
;	sts	RXFLAG,r17
	subi	r16,0x30
	lds	r17,RTCTAB+1
	add	r16,r17
        sts     RTCTAB+1,r16
        
        ;Sekunden und Control Byte 
        clr	r16
        sts     RTCTAB+0,r16
        ldi     Tmp1,0x10	;Control Byte SQW ein (1Hz)
        sts     RTCTAB+7,r16 
        ;RTC sofort setzen um Verzoegerungen zu vermeiden
        rcall   RTCSND		;RTC setzen        
        sts	RXFLAG,Zero	;RXFLAG zuruecksetzen
        rjmp	ISRRXR         

ISRRXR:	pop	r17
	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti    		;zurueck und Interrupts wieder freigeben
.ENDIF    


;-------------------------------------------------------------------------------
;	"command list" ausgeben          
;-------------------------------------------------------------------------------
.IF Flag_E232CL == 1
E232CL:	push	r16
		
	ldi	ZL,LOW(TEXT1*2)	;Tabellenzeiger initialisieren
	ldi	ZH,HIGH(TEXT1*2)
	input   r16,UCSRB	;Interrupt (UDRIE) freischalten
      	ori	r16,0x20         
      	output  UCSRB,r16        	
      	
       	pop	r16
	ret


TEXT1:	.DB	"command list:",0x0D
	.DB	"?  help",0x0D
	.DB	"g  get data (1 set)",0x0D
	.DB	"a  get all data",0x0D
	.DB	"t  set RTC ",0x0D
	.DB	"   format:  t+space+date(ddmmjj)+time(hhmm)",0x0D
	.DB	"   example: t 2204121522",0x0D,0x0A,0,0
.ENDIF    
;-------------------------------------------------------------------------------
;	Leeren des Empfangsbuffers (to flush = ausbluten, ausspuelen)
;-------------------------------------------------------------------------------
.IF Flag_E232FL == 1
E232FL: push	r16
E232F1:	input	r16,UCSRA
	sbrs	r16,RXC
	rjmp	E232FR	
	input 	r16,UDR
	rjmp 	E232F1
E232FR: pop	r16
	ret
.ENDIF    
;-------------------------------------------------------------------------------
;	Leeren des Empfangsbuffers 2 Schnittstelle (mega644p)
;-------------------------------------------------------------------------------
.IF Flag_E232FL2 == 1
E232FL2: push	r16
E232F21: input	r16,UCSR1A
	sbrs	r16,RXC1
	rjmp	E232F2R	
	input 	r16,UDR1
	rjmp 	E232F21
E232F2R: pop	r16
	ret
.ENDIF    


